/*
 * IconManager.cs
 *
 * A C# program that finds the location and image of the icons on the desktop.
 *
 * Bongshin Lee, April 2002
 *
 */

namespace IconManager
{
    using System;
    using System.Text;
    using System.Drawing;
    using System.Windows.Forms;
    using System.Runtime.InteropServices;

    // Shell class encapsulate interfaces for shell programming
    public class Shell {
        [Flags()]
        public enum SHCONTF {
            SHCONTF_FOLDERS = 0x20,
            SHCONTF_NONFOLDERS = 0x40,
            SHCONTF_INCLUDEHIDDEN = 0x80,
            SHCONTF_INIT_ON_FIRST_NEXT = 0x100,
            SHCONTF_NETPRINTERSRCH = 0x200,
            SHCONTF_SHAREABLE = 0x400,
            SHCONTF_STORAGE = 0x800
        }
  
        [Flags()]
        public enum SFGAOF {
            SFGAO_CANCOPY = 0x1,
            SFGAO_CANMOVE = 0x2,
            SFGAO_CANLINK = 0x4,
            SFGAO_STORAGE = 0x8,
            SFGAO_CANRENAME = 0x10,
            SFGAO_CANDELETE = 0x20,
            SFGAO_HASPROPSHEET = 0x40,
            SFGAO_DROPTARGET = 0x100,
            SFGAO_CAPABILITYMASK = 0x177,
            SFGAO_ENCRYPTED = 0x2000,
            SFGAO_ISSLOW = 0x4000,
            SFGAO_GHOSTED = 0x8000,
            SFGAO_LINK = 0x10000,
            SFGAO_SHARE = 0x20000,
            SFGAO_READONLY = 0x40000,
            SFGAO_HIDDEN = 0x80000,
            SFGAO_DISPLAYATTRMASK = 0xFC000,
            SFGAO_FILESYSANCESTOR = 0x10000000,
            SFGAO_FOLDER = 0x20000000,
            SFGAO_FILESYSTEM = 0x40000000,
            SFGAO_HASSUBFOLDER = unchecked( (int) 0x80000000 ),
            SFGAO_CONTENTSMASK = unchecked( (int) 0x80000000 ),
            SFGAO_VALIDATE = 0x1000000,
            SFGAO_REMOVABLE = 0x2000000,
            SFGAO_COMPRESSED = 0x4000000,
            SFGAO_BROWSABLE = 0x8000000,
            SFGAO_NONENUMERATED = 0x100000,
            SFGAO_NEWCONTENT = 0x200000,
            SFGAO_CANMONIKER = 0x400000,
            SFGAO_HASSTORAGE = 0x400000,
            SFGAO_STREAM = 0x400000,
            SFGAO_STORAGEANCESTOR = 0x800000,
            SFGAO_STORAGECAPMASK = 0x70C50008
        }

        [Flags()]
        public enum SHGNO {  
            SHGDN_NORMAL = 0x0,
            SHGDN_INFOLDER = 0x1,
            SHGDN_FOREDITING = 0x1000,
            SHGDN_FORADDRESSBAR = 0x4000, 
            SHGDN_FORPARSING = 0x8000,
        }

        [Flags()]
        public enum STRTYPE {
            STRRET_WSTR = 0x0000,
            STRRET_OFFSET = 0x0001,
            STRRET_CSTR = 0x0002
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct STRRET {
            public STRTYPE uType;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst=256)] public String str;
        }

        [ ComImport(), 
          Guid("000214E6-0000-0000-C000-000000000046"),
          InterfaceType(ComInterfaceType.InterfaceIsIUnknown) ]
        public interface IShellFolder {
            [PreserveSig()]
            uint ParseDisplayName(
                IntPtr hwnd, 
                IntPtr pbc, 
                [In(), MarshalAs(UnmanagedType.LPWStr)] string pszDisplayName,
                out uint pchEaten, 
                out IntPtr ppidl, 
                ref uint pdwAttributes);

            [PreserveSig()]
            uint EnumObjects(
                IntPtr hwnd, 
                SHCONTF grfFlags, 
                out IEnumIDList ppenumIDList);

            [PreserveSig()]
            uint BindToObject(
                IntPtr pidl, 
                IntPtr pbc, 
                [In()] ref Guid riid, 
                [MarshalAs(UnmanagedType.Interface)] out object ppv);

            [PreserveSig()]        
            uint BindToStorage(
                IntPtr pidl, 
                IntPtr pbc, 
                [In()] ref Guid riid, 
                [MarshalAs(UnmanagedType.Interface)] out object ppv);

            [PreserveSig()]    
            uint CompareIDs(
                int lParam, 
                IntPtr pidl1, 
                IntPtr pidl2);

            [PreserveSig()]    
            uint CreateViewObject(
                IntPtr hwndOwner, 
                [In()] ref Guid riid, 
                [MarshalAs(UnmanagedType.Interface)] out object ppv);

            [PreserveSig()]
            uint GetAttributesOf(
                int cidl, 
                [In(), MarshalAs(UnmanagedType.LPArray)] IntPtr[] apidl,
                [MarshalAs(UnmanagedType.LPArray)] SFGAOF[] rgfInOut);

            [PreserveSig()]
            uint GetUIObjectOf(
                IntPtr hwndOwner, 
                int cidl, 
                [In(), MarshalAs(UnmanagedType.LPArray)] IntPtr[] apidl,
                [In()] ref Guid riid, 
                IntPtr rgfReserved,
                [MarshalAs(UnmanagedType.Interface)] out object ppv);

            [PreserveSig()]
            uint GetDisplayNameOf(
                IntPtr pidl,
                SHGNO uFlags,
                out STRRET pName);

            [PreserveSig()]
            uint SetNameOf(
                IntPtr hwnd, 
                IntPtr pidl, 
                [In(), MarshalAs(UnmanagedType.LPWStr)] string pszName, 
                SHGNO uFlags, 
                out IntPtr ppidlOut);
            }

        [ ComImport(), 
          Guid("000214F2-0000-0000-C000-000000000046"),
          InterfaceType(ComInterfaceType.InterfaceIsIUnknown) ]
        public interface IEnumIDList {
            [PreserveSig()]
            uint Next(
                int celt,
                [MarshalAs(UnmanagedType.LPArray)] IntPtr[] rgelt,
                out int pceltFetched);

            void Skip(int celt);

            void Reset();

            void Clone(out IEnumIDList ppenum);
        }

        [ ComImport(),
          Guid("000214EB-0000-0000-C000-000000000046"),
          InterfaceType(ComInterfaceType.InterfaceIsIUnknown) ]
        public interface IExtractIcon {
            [PreserveSig()]
            int GetIconLocation(
                int uFlags,
                StringBuilder szFile,
                int cchMax,
                out int index,
                out int flags);
            int Extract(
                String szFile,
                int iconindex,
                out IntPtr iconLarge,
                out IntPtr iconSmall,
                long iconSize);
        }
    }

    public class Shell2 {
        [StructLayout(LayoutKind.Sequential)]
        public struct STRRET {
            public Shell.STRTYPE uType;
            [MarshalAs(UnmanagedType.LPWStr)] public string str;
        }

        [ ComImport(), 
          Guid("000214E6-0000-0000-C000-000000000046"),
          InterfaceType(ComInterfaceType.InterfaceIsIUnknown) ]
        public interface IShellFolder {
            [PreserveSig()]
            uint ParseDisplayName(
                IntPtr hwnd, 
                IntPtr pbc, 
                [In(), MarshalAs(UnmanagedType.LPWStr)] string pszDisplayName,
                out uint pchEaten, 
                out IntPtr ppidl, 
                ref uint pdwAttributes);

            [PreserveSig()]
            uint EnumObjects(
                IntPtr hwnd, 
                Shell.SHCONTF grfFlags, 
                out Shell.IEnumIDList ppenumIDList);

            [PreserveSig()]
            uint BindToObject(
                IntPtr pidl, 
                IntPtr pbc, 
                [In()] ref Guid riid, 
                [MarshalAs(UnmanagedType.Interface)] out object ppv);

            [PreserveSig()]        
            uint BindToStorage(
                IntPtr pidl, 
                IntPtr pbc, 
                [In()] ref Guid riid, 
                [MarshalAs(UnmanagedType.Interface)] out object ppv);

            [PreserveSig()]    
            uint CompareIDs(
                int lParam, 
                IntPtr pidl1, 
                IntPtr pidl2);

            [PreserveSig()]    
            uint CreateViewObject(
                IntPtr hwndOwner, 
                [In()] ref Guid riid, 
                [MarshalAs(UnmanagedType.Interface)] out object ppv);

            [PreserveSig()]
            uint GetAttributesOf(
                int cidl, 
                [In(), MarshalAs(UnmanagedType.LPArray)] IntPtr[] apidl,
                [MarshalAs(UnmanagedType.LPArray)] Shell.SFGAOF[] rgfInOut);

            [PreserveSig()]
            uint GetUIObjectOf(
                IntPtr hwndOwner, 
                int cidl, 
                [In(), MarshalAs(UnmanagedType.LPArray)] IntPtr[] apidl,
                [In()] ref Guid riid, 
                IntPtr rgfReserved,
                [MarshalAs(UnmanagedType.Interface)] out object ppv);

            [PreserveSig()]
            uint GetDisplayNameOf(
                IntPtr pidl, 
                Shell.SHGNO uFlags,
                out STRRET pName);

            [PreserveSig()]
            uint SetNameOf(
                IntPtr hwnd, 
                IntPtr pidl, 
                [In(), MarshalAs(UnmanagedType.LPWStr)] string pszName, 
                Shell.SHGNO uFlags, 
                out IntPtr ppidlOut);
        }

        [ ComImport(),
          Guid("000214FA-0000-0000-C000-000000000046"),
          InterfaceType(ComInterfaceType.InterfaceIsIUnknown) ]
        public interface IExtractIcon {
            [PreserveSig()]
            int GetIconLocation(
                int uFlags,
                [Out(), MarshalAs(UnmanagedType.LPWStr)] string str,
                int cchMax,
                out int index,
                out int flags);
            int Extract(
                String szFile,
                int iconindex,
                out IntPtr iconLarge,
                out IntPtr iconSmall,
                long iconSize);
        }
    }

    // Win32API class contains necessary declaration of the function, data types and constants
    public class Win32API
    {
        [DllImport("user32")]
        public static extern int SendMessage(IntPtr hWnd, uint msg, IntPtr lparam, IntPtr rparam);

        [DllImport("user32")]
        public static extern IntPtr SetFocus(IntPtr hWnd);

        [DllImport("user32")]
        public static extern IntPtr FindWindow(String pClass, String pWindow);

        [DllImport("user32")]
        public static extern IntPtr FindWindowEx(IntPtr hParent, IntPtr hChildAfter, String pClass, String pWindow);

        [DllImport("user32")]
        public static extern int GetWindowThreadProcessId(IntPtr hWnd, ref int pid);

        [DllImport("kernel32")]
        public static extern IntPtr OpenProcess(int access, bool bInherit, int pid);

        [DllImport("kernel32")]
        public static extern IntPtr VirtualAllocEx(IntPtr hProcess, IntPtr addr, int size, int alloc, int protect);

        [DllImport("kernel32")]
        public static extern bool VirtualFreeEx(IntPtr hProcess, IntPtr addr, int size, int freetype);

        [DllImport("kernel32")]
        public static extern bool WriteProcessMemory(IntPtr hProcess, IntPtr baddr, IntPtr buffer, int size, ref int written);

        [DllImport("kernel32")]
        public static extern bool ReadProcessMemory(IntPtr hProcess, IntPtr baddr, IntPtr buffer, int size, ref int written);

        [DllImport("kernel32")]
        public static extern bool CloseHandle(IntPtr hProcess);

        [DllImport("shell32")]
        public static extern IntPtr SHGetDesktopFolder(out Shell.IShellFolder folder);

        [DllImport("shell32")]
        public static extern IntPtr SHGetDesktopFolder(out Shell2.IShellFolder folder);

        [DllImport("shell32")]
        public static extern IntPtr SHGetFileInfo(String path, int attr, ref SHFILEINFO fi, int cb, int flags);

        [StructLayout(LayoutKind.Sequential)]
        public struct POINT {
            public int x;
            public int y;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct LV_ITEM {
            public int mask;
            public int iItem;
            public int iSubItem;
            public int state;
            public int stateMask;
            public IntPtr pszText;
            public int cchTextMax;
            public int iImage;
            public IntPtr lParam;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct SHFILEINFO {
            public IntPtr hIcon;
            public int iIcon;
            public int attr;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst=256)] public String dispName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst=80)] public String typeName;
        }

        // From CommCtrl.h
        public const int LVM_FIRST = 0x1000;
	public const int LVM_GETIMAGELIST = LVM_FIRST + 2;
        public const int LVM_GETITEMCOUNT = LVM_FIRST + 4;
        public const int LVM_SETITEMPOSITION = LVM_FIRST + 15;
        public const int LVM_GETITEMPOSITION = LVM_FIRST + 16;
        public const int LVM_SETITEMSTATE = LVM_FIRST + 43;
        public const int LVM_GETITEMTEXT = LVM_FIRST + 45;
        public const int LVIF_TEXT = 0x1;
        public const int LVIF_IMAGE = 0x2;
        public const int LVIF_STATE = 0x8;
        public const int LVIS_FOCUSED = 0x1;
        public const int LVIS_SELECTED = 0x2;

        public const int PROCESS_VM_OPERATION = 0x0008;
        public const int PROCESS_VM_READ = 0x0010;
        public const int PROCESS_VM_WRITE = 0x0020;
        public const int PAGE_READWRITE = 0x04;
        public const int MEM_COMMIT = 0x1000;
        public const int MEM_RESERVE = 0x2000;
        public const int MEM_RELEASE = 0x8000;

        // From ShellApi.h
        public const int SHGFI_LARGEICON = 0x0;
        public const int SHGFI_SMALLICON = 0x1;
        public const int SHGFI_ICON = 0x0100;
        public const int SHGFI_LINKOVERLAY = 0x8000;  
    }

    public class IconLocator {
        public struct ICON {
            public Point pt;		// position in pixel
            public Icon iconImg;	// icon image
            public String name;		// icon name
            public bool shortcut;	// boolean flag to specify whether the icon is a shortcut
        }

        private Icon[] icons;
        private Win32API.POINT[] pts;
        private String[] name1;
        private String[] name2;
        private int[] namelen;
        private int[] index;
        private bool[] shortcut;
        private int maxLen;
        private IntPtr hDesk;
        private IntPtr hProgman;

        // Find the icon position
        private unsafe void FindPosition(int count) {
            int pid = 0;
            int tid = Win32API.GetWindowThreadProcessId(hDesk, ref pid);
            IntPtr hProcess = Win32API.OpenProcess(Win32API.PROCESS_VM_OPERATION | Win32API.PROCESS_VM_READ | Win32API.PROCESS_VM_WRITE, false, pid);
            if (hProcess.Equals(0)) {
                Console.WriteLine("Can't Open Process");
            }

            Win32API.POINT pt = new Win32API.POINT();
            int size = Marshal.SizeOf(pt);

            // We have to allocate virtual memory space in the explorer memory
            IntPtr shared = Win32API.VirtualAllocEx(hProcess, IntPtr.Zero, size, Win32API.MEM_RESERVE | Win32API.MEM_COMMIT, Win32API.PAGE_READWRITE);

            int read = 0;
            for (int i = 0; i < count; i++) {
                // Send message to the listview in the desktop to get the item position
                // Ask the listview to write the result in the shared memory 
	        Win32API.SendMessage(hDesk, Win32API.LVM_GETITEMPOSITION, new IntPtr(i), shared);

                // read the result from the shared memory
	        Win32API.ReadProcessMemory(hProcess, shared, new IntPtr(&pt), size, ref read);
		pts[i].x = pt.x;
		pts[i].y = pt.y;
	    }

	    if (!Win32API.VirtualFreeEx(hProcess, shared, 0, Win32API.MEM_RELEASE)) {
	        Console.WriteLine("Can't Free Memroy");
            }

            Win32API.LV_ITEM lvi = new Win32API.LV_ITEM();
            lvi.iSubItem = 0;
            lvi.cchTextMax = maxLen + 1;
            size = Marshal.SizeOf(lvi);

            shared = Win32API.VirtualAllocEx(hProcess, IntPtr.Zero, size, Win32API.MEM_RESERVE | Win32API.MEM_COMMIT, Win32API.PAGE_READWRITE);
            IntPtr shared2 = Win32API.VirtualAllocEx(hProcess, IntPtr.Zero, 256, Win32API.MEM_RESERVE | Win32API.MEM_COMMIT, Win32API.PAGE_READWRITE);
            lvi.pszText = shared2;
        
            int written = 0;
            for (int i = 0; i < count; i++) {
                if (!Win32API.WriteProcessMemory(hProcess, shared, new IntPtr(&lvi), size, ref written)) {
                    Console.WriteLine("Can't Write2");
                }

	        Win32API.SendMessage(hDesk, Win32API.LVM_GETITEMTEXT, new IntPtr(i), shared);

                // Unfortunately, I cannot get the icon name in an elegant manner
                // If I try to get the name at once, it returns wrong string.
                StringBuilder sb = new StringBuilder();
                char[] n = new char[1];
                fixed(char *p = n) {
                    IntPtr addr = new IntPtr(p);
                    int baseaddr = shared2.ToInt32();
                    for (int offset = 0; offset < maxLen + 1; offset++) {
		        if (!Win32API.ReadProcessMemory(hProcess, new IntPtr(baseaddr + offset), addr, 1, ref read)) {
		            Console.WriteLine("Can't Read");
                        }
                        sb.Insert(offset, p[0]);
                    }
                }
                name2[i] = sb.ToString();
	    }

	    if (!Win32API.VirtualFreeEx(hProcess, shared, 0, Win32API.MEM_RELEASE)) {
	        Console.WriteLine("Can't Free2");
            }
	    if (!Win32API.VirtualFreeEx(hProcess, shared2, 0, Win32API.MEM_RELEASE)) {
	        Console.WriteLine("Can't Free22");
            }

            if (!Win32API.CloseHandle(hProcess)) {
                Console.WriteLine("Can't close handle");
            }
        }

        // Find the handle of the listview in the desktop
        private IntPtr GetSysLVHwnd() {
            hProgman = Win32API.FindWindow("Progman", null);
            IntPtr h = Win32API.FindWindowEx(hProgman, new IntPtr(0), "SHELLDLL_defVIEW", null);
            return Win32API.FindWindowEx(h, new IntPtr(0), "SysListView32", null);
        }

        // move an icon to the new position
        public void Move(int idx, Point pt) {
            long pos = pt.Y;
            pos = (pos << 16) + pt.X;

            if (Win32API.SendMessage(hDesk, Win32API.LVM_SETITEMPOSITION, new IntPtr(index[idx]), new IntPtr(pos)) == 0) {
                Console.WriteLine("Can't set position");
            }
        }

        // Although I implemented this function, we cannot see any effect.
        // Since the caller program (ex. IconView) is active, we cannot see the selected icon.
        public unsafe void Select(int idx) {
            int pid = 0;
            int tid = Win32API.GetWindowThreadProcessId(hDesk, ref pid);
            IntPtr hProcess = Win32API.OpenProcess(Win32API.PROCESS_VM_OPERATION | Win32API.PROCESS_VM_READ | Win32API.PROCESS_VM_WRITE, false, pid);
            if (hProcess.Equals(0)) {
                Console.WriteLine("Can't Open Process");
            }

            Win32API.LV_ITEM lvi = new Win32API.LV_ITEM();
            int size = Marshal.SizeOf(lvi);
            lvi.mask = Win32API.LVIF_STATE;
            lvi.state = 0;
            lvi.stateMask = Win32API.LVIS_SELECTED | Win32API.LVIS_FOCUSED;

            IntPtr shared = Win32API.VirtualAllocEx(hProcess, IntPtr.Zero, size, Win32API.MEM_RESERVE | Win32API.MEM_COMMIT, Win32API.PAGE_READWRITE);
            int written = 0;
            if (!Win32API.WriteProcessMemory(hProcess, shared, new IntPtr(&lvi), size, ref written)) {
                Console.WriteLine("Can't Write2");
            }

            Win32API.SendMessage(hDesk, Win32API.LVM_SETITEMSTATE, new IntPtr(-1), shared);

            lvi.state = Win32API.LVIS_SELECTED | Win32API.LVIS_FOCUSED;
            if (!Win32API.WriteProcessMemory(hProcess, shared, new IntPtr(&lvi), size, ref written)) {
                Console.WriteLine("Can't Write2");
            }
            Win32API.SendMessage(hDesk, Win32API.LVM_SETITEMSTATE, new IntPtr(index[idx]), shared);

	    if (!Win32API.VirtualFreeEx(hProcess, shared, 0, Win32API.MEM_RELEASE)) {
	        Console.WriteLine("Can't Free2");
            }

            if (!Win32API.CloseHandle(hProcess)) {
                Console.WriteLine("Can't close handle");
            }

            Win32API.SetFocus(hDesk);
            Win32API.SetFocus(hProgman);
        }

        // locate icons on the desktop
        public ICON[] LocateIcons() {
            hDesk = GetSysLVHwnd();	// get the handle of the listview in the desktop
            if (hDesk.Equals(0)) {	
                return null;
            }

            // get the number of icons on the desktop
            int count = Win32API.SendMessage(hDesk, Win32API.LVM_GETITEMCOUNT, IntPtr.Zero, IntPtr.Zero);
            if (count == 0) {
                return null;
            }

            name1 = new String[count];		// icon names getting from the shell (desktop folder)
            name2 = new String[count];		// icon names getting from the listview in the desktop
            namelen = new int[count];
            shortcut = new bool[count];
            pts = new Win32API.POINT[count];
            icons = new Icon[count];
            
            // retrieve icons from the desktop folder
            // we may not be able to get all the icons for various reasons
            int valid_icons = ExtractIcons();

            // retrieve positions of icons
            FindPosition(count);

            // we have to map above two info by the icon name
            Map();

            // build return value: array of ICONs
            ICON[] ic = new ICON[valid_icons];
            for (int i = 0; i < valid_icons; i++) {
                ic[i].pt.X = pts[i].x;
                ic[i].pt.Y = pts[i].y;
                ic[i].iconImg = icons[i];
                ic[i].name = name1[i];
                ic[i].shortcut = shortcut[i];
            }

            return ic;            // return the result
        }

        // Retrives icon using file path        
        public Icon GetIcon(String path, bool large) {
            Win32API.SHFILEINFO fi = new Win32API.SHFILEINFO();
            int size = Marshal.SizeOf(fi);
            int flags = Win32API.SHGFI_ICON;
            if (large) {
                flags |= Win32API.SHGFI_LARGEICON;
            } else {
                flags |= Win32API.SHGFI_SMALLICON;
            } 
            
            Win32API.SHGetFileInfo(path, 0, ref fi, size, flags);
            
            if (fi.hIcon.Equals(0)) {
                return null;
            } else {
                // Create Icon using the handle
                Icon ic;
                if (large) {
                    ic = new Icon(Icon.FromHandle(fi.hIcon), new Size(32,32));
                } else {
                    ic = new Icon(Icon.FromHandle(fi.hIcon), new Size(16,16));
                }
                return ic;
            }
        }

        private void Map() {
            int longest;
            int j;

            index = new int[name1.Length];
            for (int i = 0; i < name1.Length; i++) {
                index[i] = i;
            }

            for (int i = 0; i < name1.Length; i++) {
                longest = 0;
                for (j = 1; j < name1.Length; j++) {
                    if (namelen[j] > namelen[longest]) {
                        longest = j;
                    }
                }

                for (j = 0; j < name2.Length; j++) {
                    if (name2[j].StartsWith(name1[longest])) {
                        break;
                    }
                }

                if (j < name2.Length) {
                    int t = index[longest];
                    index[longest] = index[j];
                    index[j] = t;

                    Win32API.POINT temp = pts[longest];
                    pts[longest] = pts[j];
                    pts[j] = temp;

                    namelen[longest] = 0;
                    name2[j] = name2[longest];
                    name2[longest] = "";
                }
            }
        }

        // Retrieve icon's image and name on the desktop
        private int ExtractIcons() {        
            Shell.IShellFolder ISFolder;
            Shell.IEnumIDList IEidList;
            Shell.IExtractIcon IEIcon;
            Shell.STRRET strtName;

            Shell2.IShellFolder ISFolder2;
            Shell.IEnumIDList IEidList2;
            Shell2.STRRET strtName2;

            IntPtr[] idlist = new IntPtr[1];
            IntPtr[] idlist2 = new IntPtr[1];
            int celt, count = 0;
         
            // Get a pointer to the IShellFolder interface of the desktop folder   
            IntPtr hr = Win32API.SHGetDesktopFolder(out ISFolder);
            hr = Win32API.SHGetDesktopFolder(out ISFolder2);
            
            uint h = ISFolder.EnumObjects(IntPtr.Zero, Shell.SHCONTF.SHCONTF_FOLDERS | Shell.SHCONTF.SHCONTF_NONFOLDERS, out IEidList);
            h = ISFolder2.EnumObjects(IntPtr.Zero, Shell.SHCONTF.SHCONTF_FOLDERS | Shell.SHCONTF.SHCONTF_NONFOLDERS, out IEidList2);

            maxLen = 0;
            String iconName;
            while ((h = IEidList.Next(1, idlist, out celt)) != 1 && (celt == 1)) {
                IEidList2.Next(1, idlist2, out celt);

                Shell.SFGAOF[] rgf = new Shell.SFGAOF[1];
                try {
                    // Call IShellFolder::GetDisplayNameOf to get the icon name
                    ISFolder.GetDisplayNameOf(idlist[0], Shell.SHGNO.SHGDN_NORMAL, out strtName);
                    rgf[0] = Shell.SFGAOF.SFGAO_LINK;
                    if (strtName.uType == Shell.STRTYPE.STRRET_CSTR) {
                        // Call IShellFolder::GetAttributesOf to get the icon property
                        ISFolder.GetAttributesOf(1, idlist, rgf);
                        iconName = strtName.str;
                    } else if (strtName.uType == Shell.STRTYPE.STRRET_WSTR) {
                        ISFolder2.GetDisplayNameOf(idlist2[0], Shell.SHGNO.SHGDN_NORMAL, out strtName2);
                        ISFolder2.GetAttributesOf(1, idlist2, rgf);
                        iconName = strtName2.str;
                    } else {
                        Console.WriteLine("Can't get the icon name");
                        continue;
                    }
                } catch (Exception e) {
                    Console.WriteLine(e.ToString());
                    continue;
                }

                Console.WriteLine(iconName);	// Print the icon name for debugging
                try {
                    object o = new object();
                    Guid id = new Guid("000214EB-0000-0000-C000-000000000046");
                    // Call IShellFolder::GetUIObjectOf to get the IExtractIcon interface
                    if (ISFolder.GetUIObjectOf(IntPtr.Zero, 1, idlist, ref id, IntPtr.Zero, out o) == 0) {
                        IEIcon = (Shell.IExtractIcon)o;
                        StringBuilder str2 = new StringBuilder(1024);
                        int index, flags;
                        // Call IExtractIcon::GetIconLocation to retrieve the icon's location (this is not a position);
                        if ((IEIcon.GetIconLocation(0, str2, 1024, out index, out flags)) != 1) {
                            IntPtr hIcon = new IntPtr();
                            IntPtr hIcon2 = new IntPtr();
                            long iconSize = 16;
                            iconSize = (iconSize << 16) | 32;

                            // Call IExtractIcon::Extract to retrive the icon's handle
                            if ((IEIcon.Extract(str2.ToString(), index, out hIcon, out hIcon2, iconSize)) != 1) {
                                if (hIcon.Equals(IntPtr.Zero)) {
                                    Console.WriteLine("Can't get icon handle");
                                } else {
                                    // Create Icon using the handle
                                    Icon ic = new Icon(Icon.FromHandle(hIcon), new Size(32,32));
                                    icons[count] = ic;

                                    // set the shortcut flag
                                    if (rgf[0] == Shell.SFGAOF.SFGAO_LINK) {
                                        shortcut[count] = true;
                                    } else {
                                        shortcut[count] = false;
                                    }

                                    if (iconName.Length > maxLen) {
                                        maxLen = iconName.Length;
                                    }
                                    namelen[count] = iconName.Length;
                                    name1[count++] = iconName;
                                }
                            } else {
                                Console.WriteLine("Can't extract Icon");
                            }
                        } else {
                            Console.WriteLine("Can't get Icon location");
                        }
                    } else {
                      Console.WriteLine("Can't get IExtractIcon interface");
                    }
                } catch (Exception e) {
                    Console.WriteLine(e.ToString());
                }
            }

            return count;
        }
    }
}

