/*
 * IconView.cs
 *
 * A C# program that shows the icons on the desktop using IconManager.
 *
 * Bongshin Lee, April 2002
 */

using System;
using System.Drawing;
using System.Windows.Forms;
using IconManager;		// Include IconManager

class IconView : Form {
    IconLocator loc = new IconLocator();
    IconLocator.ICON[] icons;
    IconCanvas iCanvas;

    public static void Main() {
        Application.Run(new IconView());
    }

    public IconView() {
        Text = "Icon Locator Test Program";

        icons = loc.LocateIcons();	// create an IconLocator
        Icon largeIcon = loc.GetIcon("IconView.exe", true);	// retrieve large icon (32x32)
        Icon smallIcon = loc.GetIcon("IconView.exe", false);	// retrieve small icon (16x16)

        iCanvas = new IconCanvas(icons, largeIcon, smallIcon);
        iCanvas.Size = new Size(700, 700);
        iCanvas.MouseDown += new MouseEventHandler(onMouseDown);
        Controls.Add(iCanvas);
        ClientSize = iCanvas.Size;
    }

    private void onMouseDown(object sender, MouseEventArgs e) {
        int col = (e.X - 10) / 100;
        int row = (e.Y - 10) / 80; 
        int idx = row * 7 + col;

        if (idx < icons.Length) {
            if (e.Button == MouseButtons.Left) {
                icons[idx].pt.Offset(10, 10);
            } else {
                icons[idx].pt.Offset(-10, -10);
            }
            loc.Move(idx, icons[idx].pt);	// Move an icon to the new position
            iCanvas.Invalidate();
        }
    }

    class IconCanvas : Control {
        IconLocator.ICON[] icons;
        Icon largeIcon;
        Icon smallIcon;
        static Font font = new Font("Helvetica", 10);
        static SolidBrush bBrush = new SolidBrush(Color.Black);
        static SolidBrush rBrush = new SolidBrush(Color.Red);

        public IconCanvas(IconLocator.ICON[] icons, Icon lIcon, Icon sIcon) {
            this.icons = icons;
            this.largeIcon = lIcon;
            this.smallIcon = sIcon;

            SetStyle(System.Windows.Forms.ControlStyles.DoubleBuffer, true);
        }

        protected override void OnPaint(PaintEventArgs e) {
            Graphics g = e.Graphics;
            BackColor = Color.White;

            if (icons == null) {
                return;
            }

            int col = 0;
            int row = 0;
            // enumerate icons
            for (int i = 0; i < icons.Length; i++) {
                if (col * 100 + 100> Width) {
                    col = 0;
                    row++;
                }
                g.DrawIcon(icons[i].iconImg, 10+col*100, 10+row*80);
                String pos = "(" + icons[i].pt.X.ToString() + ", " + icons[i].pt.Y.ToString() + ")";
                g.DrawString(icons[i].name, font, bBrush, 10.0F+col*100, 45.0F+row*80);
                g.DrawString(pos, font, bBrush, 10.0F+col*100, 65.0F+row*80);
                if (icons[i].shortcut) {
                    g.DrawString("S", font, rBrush, 50.0F+col*100, 10+row*80);
                }
                col++;
            }

            // draw large and small icons
            if (largeIcon != null) {
                g.DrawIcon(largeIcon, 10, 130+row*80);
            }
            if (smallIcon != null) {
                g.DrawIcon(smallIcon, 50, 130+row*80);
            }
        }
    }
}
